include_guard(GLOBAL)

if (ENABLE_SEGGER_RTT)
	set(SEGGER_RTT_UP_BUFFERS 3 CACHE STRING "Segger RTT: max number of up-buffers")
	set(SEGGER_RTT_UP_BUFFER_SIZE 256 CACHE STRING "Segger RTT: size of the up-buffer")
	set(SEGGER_RTT_DOWN_BUFFERS 3 CACHE STRING "Segger RTT: max number of down-buffers")
	set(SEGGER_RTT_DOWN_BUFFER_SIZE 16 CACHE STRING "Segger RTT: size of the down-buffer")
	set(SEGGER_RTT_MODE skip CACHE STRING "Segger RTT mode: skip, trim, block")
	set(SEGGER_RTT_SECTION default CACHE STRING "Segger RTT section: default, .data.segger_rtt")
	
	message("--  Segger RTT : enabled")
	if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/RTT/SEGGER_RTT.c AND EXISTS $ENV{SEGGER_RTT_SRC})
		set(SEGGER_RTT_SRC "$ENV{SEGGER_RTT_SRC}")
		message("--  Extract SEGGER RTT Sources ${SEGGER_RTT_SRC}")
		file(ARCHIVE_EXTRACT
			INPUT ${SEGGER_RTT_SRC}
			PATTERNS "RTT"
			DESTINATION "${CMAKE_CURRENT_SOURCE_DIR}"
		)
		file(ARCHIVE_EXTRACT
			INPUT ${SEGGER_RTT_SRC}
			DESTINATION "${CMAKE_CURRENT_SOURCE_DIR}/.edx/SEGGER_RTT"
		)
		if (EXISTS "${CMAKE_CURRENT_SOURCE_DIR}/.edx/SEGGER_RTT/Config/SEGGER_RTT_Conf.h")
			file(COPY "${CMAKE_CURRENT_SOURCE_DIR}/.edx/SEGGER_RTT/Config/SEGGER_RTT_Conf.h" DESTINATION "${CMAKE_CURRENT_SOURCE_DIR}/RTT")
		endif()
		if (EXISTS "${CMAKE_CURRENT_SOURCE_DIR}/.edx/SEGGER_RTT/Syscalls/SEGGER_RTT_Syscalls_GCC.c")
			file(COPY "${CMAKE_CURRENT_SOURCE_DIR}/.edx/SEGGER_RTT/Syscalls/SEGGER_RTT_Syscalls_GCC.c" DESTINATION "${CMAKE_CURRENT_SOURCE_DIR}/RTT")
		endif()
		file(REMOVE_RECURSE "${CMAKE_CURRENT_SOURCE_DIR}/.edx/SEGGER_RTT")
	endif()
	if (EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/RTT/SEGGER_RTT.c)
		file(GLOB_RECURSE SEGGER_RTT_SOURCES
			LIST_DIRECTORIES false
			${CMAKE_CURRENT_SOURCE_DIR}/RTT/*.c
			${CMAKE_CURRENT_SOURCE_DIR}/RTT/*.s
		)
		add_compile_options(-I${CMAKE_CURRENT_SOURCE_DIR}/RTT)

		set(SEGGER_RTT_DEFINITIONS
			SEGGER_RTT_MAX_NUM_UP_BUFFERS=${SEGGER_RTT_UP_BUFFERS}
			BUFFER_SIZE_UP=${SEGGER_RTT_UP_BUFFER_SIZE}
			SEGGER_RTT_MAX_NUM_DOWN_BUFFERS=${SEGGER_RTT_DOWN_BUFFERS}
			BUFFER_SIZE_DOWN=${SEGGER_RTT_DOWN_BUFFER_SIZE}
		)

		if (NOT "${SEGGER_RTT_SECTION}" STREQUAL "default")
			list(APPEND SEGGER_RTT_DEFINITIONS SEGGER_RTT_SECTION="${SEGGER_RTT_SECTION}")
		endif()

		if ("${SEGGER_RTT_MODE}" STREQUAL "skip")
			list(APPEND SEGGER_RTT_DEFINITIONS SEGGER_RTT_MODE_DEFAULT=SEGGER_RTT_MODE_NO_BLOCK_SKIP)
		elseif ("${SEGGER_RTT_MODE}" STREQUAL "trim")
			list(APPEND SEGGER_RTT_DEFINITIONS SEGGER_RTT_MODE_DEFAULT=SEGGER_RTT_MODE_NO_BLOCK_TRIM)
		elseif ("${SEGGER_RTT_MODE}" STREQUAL "block")
			list(APPEND SEGGER_RTT_DEFINITIONS SEGGER_RTT_MODE_DEFAULT=SEGGER_RTT_MODE_BLOCK_IF_FIFO_FULL)
		else()
			message("${_WARNING_PREFIX} : Invalid Segger RTT mode [${SEGGER_RTT_MODE}]!")
		endif()

		list(TRANSFORM SEGGER_RTT_SOURCES REPLACE "\\\\" "/")
		
		set_source_files_properties(
			${SEGGER_RTT_SOURCES}
			PROPERTIES COMPILE_DEFINITIONS "${SEGGER_RTT_DEFINITIONS}"
		)
		
		if ("${TARGET_SPECS}" MATCHES "^rdimon.*")
			message("${_WARNING_PREFIX} : Segger RTT is in conflict with semihosting(rdimon/rdimon-v2m)!")
		endif()
	else()
		message("${_WARNING_PREFIX} : Segger RTT was not found!")
	endif()
else()
	message("--  Segger RTT : disabled")
endif()
