include_guard(GLOBAL)

set(_IS_CUBEMX_PROJECT 0)
if (EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/.mxproject)
	# STM32CubeMX project
	set(_IS_CUBEMX_PROJECT 1)
	# _CUBEMX_LINK_SCRIPT
	# _CUBEMX_CORE_SOURCES
	# _CUBEMX_DRIVERS_SOURCES
	# _CUBEMX_STARTUP_SOURCE
	# _CUBEMX_INCLUDE_DIRS
	file(GLOB_RECURSE _CUBEMX_CORE_SOURCES
		LIST_DIRECTORIES false
		Core/*.cxx
		Core/*.cpp
		Core/*.cc
		Core/*.c
		Core/*.s
	)
	file(GLOB_RECURSE _CUBEMX_DRIVERS_SOURCES
		LIST_DIRECTORIES false
		Drivers/*.cxx
		Drivers/*.cpp
		Drivers/*.cc
		Drivers/*.c
		Drivers/*.s
	)
	
	file(GLOB_RECURSE _CUBEMX_MIDDLEWARES_SOURCES
		LIST_DIRECTORIES false
		Middlewares/*.cxx
		Middlewares/*.cpp
		Middlewares/*.cc
		Middlewares/*.c
		Middlewares/*.s
	)
	
	file(GLOB_RECURSE _CUBEMX_FATFS_SOURCES
		LIST_DIRECTORIES false
		FATFS/*.cxx
		FATFS/*.cpp
		FATFS/*.cc
		FATFS/*.c
		FATFS/*.s
	)
	
	file(GLOB_RECURSE _CUBEMX_USB_DEVICE_SOURCES
		LIST_DIRECTORIES false
		USB_DEVICE/*.cxx
		USB_DEVICE/*.cpp
		USB_DEVICE/*.cc
		USB_DEVICE/*.c
		USB_DEVICE/*.s
	)
	
	file(GLOB_RECURSE _CUBEMX_INCLUDE_DIRS
		LIST_DIRECTORIES true
		Core/.*
		Drivers/.*
		FATFS/.*
		Middlewares/.*
		USB_DEVICE/.*
	)
	
	string(TOLOWER "${_CUBEMX_INCLUDE_DIRS}" _CUBEMX_INCLUDE_DIRS)
	list(FILTER _CUBEMX_INCLUDE_DIRS INCLUDE REGEX "^.*/(inc(lude)?|cmsis_rtos(_v[0-9]+)?|gcc.arm_cm[0-9]+|fatfs.src|app|target)$")

	file(GLOB _CUBEMX_LINK_SCRIPT
		LIST_DIRECTORIES false
		RELATIVE ${CMAKE_CURRENT_SOURCE_DIR}
		STM32*.ld
	)
	file(GLOB _CUBEMX_STARTUP_SOURCE
		LIST_DIRECTORIES false
		RELATIVE ${CMAKE_CURRENT_SOURCE_DIR}
		startup_*.s
	)
	file(GLOB _CUBEMX_CONFIG
		LIST_DIRECTORIES false
		RELATIVE ${CMAKE_CURRENT_SOURCE_DIR}
		*.ioc
	)

	file(STRINGS ${CMAKE_CURRENT_SOURCE_DIR}/.mxproject _CUBEMX_CDEFINES
		LIMIT_COUNT 1
		REGEX "^\\s*CDefines\\s*=\\s*.*"
	)
	string(REGEX REPLACE "^\\s*CDefines\\s*=\\s*(.*)" "\\1" _CUBEMX_CDEFINES "${_CUBEMX_CDEFINES}")

	add_compile_definitions(${_CUBEMX_CDEFINES})

	file(STRINGS ${_CUBEMX_CONFIG} _CUBEMX_TARGET_CPU
		LIMIT_COUNT 1
		REGEX "^Mcu\\.CPN\\s*=\\s*STM32.*"
	)
	string(REGEX REPLACE "^Mcu\\.CPN\\s*=\\s*(STM32......).*" "\\1" TARGET_CPU "${_CUBEMX_TARGET_CPU}")
	string(TOLOWER "${TARGET_CPU}" TARGET_CPU)

	message("-- ${_ESC_INFO}STM32CubeMX Project${_ESC_RST}")
	message("--    SDK_TYPE : $ENV{SDK_TYPE}")
	message("-- SDK_INCLUDE : ${_CUBEMX_INCLUDE_DIRS}")
else()
	if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/sdk)
		file(MAKE_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}/sdk)
	endif()
	message("--    SDK_TYPE : $ENV{SDK_TYPE}")
	message("-- SDK_INCLUDE : $ENV{SDK_INCLUDE}")
	message("--     SDK_LIB : $ENV{SDK_LIB}")
endif()

if (NOT _IS_CUBEMX_PROJECT)
	option(ENABLE_HAL_DRIVER "Build & Link with HAL drivers" OFF)
	option(ENABLE_LL_DRIVER "Build & Link with Low Level drivers" OFF)
	option(ENABLE_NN_LIB "Build & Link with NN lib" OFF)
else()
endif()
option(ENABLE_DSP_LIB "Build & Link with DSP lib" OFF)
option(ENABLE_SEGGER_RTT "Build & Link with SEGGER RTT support, J-Link required!" OFF)
set(TARGET_CPU_FLOAT_ABI soft CACHE STRING "Select float ABI: none, soft, softfp, hard")
set(TARGET_SPECS nano CACHE STRING "Select specs: none, nosys, nano, rdimon, rdimon-v2m")
set(TARGET_HSE_VALUE auto CACHE STRING "Select HSE Speed: auto, <number>")
set(TARGET_HSI_VALUE auto CACHE STRING "Select HSI Speed: auto, <number>")

if ("${TARGET_CPU}" STREQUAL "")
	message(FATAL_ERROR "TARGET_CPU was not specified!")
endif()

string(TOLOWER "${TARGET_CPU}" TARGET_CPU)

if ("${TARGET_CPU}" MATCHES "^stm32f([0])([0-9][0-9])..$")
	if ("${TARGET_CPU}" MATCHES "^stm32f([0])(3[01]).([468bc])$")
		set(_STM_CPU_PREFIX "STM32F${CMAKE_MATCH_1}${CMAKE_MATCH_2}")
		set(_STM_CPU_SUFFIX ${CMAKE_MATCH_3})
		set(TARGET_CPU_TYPE_COMMON "STM32F${CMAKE_MATCH_1}xx")
		if ("${_STM_CPU_SUFFIX}" MATCHES "[46]")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}x6)
		elseif ("${_STM_CPU_SUFFIX}" MATCHES "[8]")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}x8)
		elseif ("${_STM_CPU_SUFFIX}" MATCHES "[bc]")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xC)
		else()
			message(FATAL_ERROR "Unsupported CPU[${TARGET_CPU}]!")
		endif()
	elseif ("${TARGET_CPU}" MATCHES "^stm32f([0])(42).([46])$")
		set(_STM_CPU_PREFIX "STM32F${CMAKE_MATCH_1}${CMAKE_MATCH_2}")
		set(_STM_CPU_SUFFIX ${CMAKE_MATCH_3})
		set(TARGET_CPU_TYPE_COMMON "STM32F${CMAKE_MATCH_1}xx")
		if ("${_STM_CPU_SUFFIX}" MATCHES "[46]")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}x6)
		else()
			message(FATAL_ERROR "Unsupported CPU[${TARGET_CPU}]!")
		endif()
	elseif ("${TARGET_CPU}" MATCHES "^stm32f([0])(51).([468])$")
		set(_STM_CPU_PREFIX "STM32F${CMAKE_MATCH_1}${CMAKE_MATCH_2}")
		set(_STM_CPU_SUFFIX ${CMAKE_MATCH_3})
		set(TARGET_CPU_TYPE_COMMON "STM32F${CMAKE_MATCH_1}xx")
		if ("${_STM_CPU_SUFFIX}" MATCHES "[468]")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}x8)
		else()
			message(FATAL_ERROR "Unsupported CPU[${TARGET_CPU}]!")
		endif()
	elseif ("${TARGET_CPU}" MATCHES "^stm32f([0])(91).([bc])$")
		set(_STM_CPU_PREFIX "STM32F${CMAKE_MATCH_1}${CMAKE_MATCH_2}")
		set(_STM_CPU_SUFFIX ${CMAKE_MATCH_3})
		set(TARGET_CPU_TYPE_COMMON "STM32F${CMAKE_MATCH_1}xx")
		if ("${_STM_CPU_SUFFIX}" MATCHES "[bc]")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xC)
		else()
			message(FATAL_ERROR "Unsupported CPU[${TARGET_CPU}]!")
		endif()
	elseif ("${TARGET_CPU}" MATCHES "^stm32f([0])(7[012]).([468bcdefg])$")
		set(_STM_CPU_PREFIX "STM32F${CMAKE_MATCH_1}${CMAKE_MATCH_2}")
		set(_STM_CPU_SUFFIX ${CMAKE_MATCH_3})
		set(TARGET_CPU_TYPE_COMMON "STM32F${CMAKE_MATCH_1}xx")
		if ("${_STM_CPU_SUFFIX}" MATCHES "[46]")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}x6)
		elseif ("${_STM_CPU_SUFFIX}" MATCHES "[8b]")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xB)
		else()
			message(FATAL_ERROR "Unsupported CPU[${TARGET_CPU}]!")
		endif()
	elseif ("${TARGET_CPU}" MATCHES "^stm32f(0..).([468bcdefg])$")
		set(TARGET_CPU_TYPE_DEF "STM32F${CMAKE_MATCH_1}xx")
		set(TARGET_CPU_TYPE_COMMON "STM32F0xx")
	else()
		message(FATAL_ERROR "Unsupported CPU[${TARGET_CPU}]!")
	endif()
elseif ("${TARGET_CPU}" MATCHES "^stm32f([1])([0-9][0-9]).([468bcdefg])$")
	set(_STM_CPU_PREFIX "STM32F${CMAKE_MATCH_1}${CMAKE_MATCH_2}")
	set(_STM_CPU_SUFFIX ${CMAKE_MATCH_3})
	set(TARGET_CPU_TYPE_COMMON "STM32F${CMAKE_MATCH_1}xx")
	if ("${_STM_CPU_SUFFIX}" MATCHES "[46]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}x6)
	elseif ("${_STM_CPU_SUFFIX}" MATCHES "[8b]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xB)
	elseif ("${_STM_CPU_SUFFIX}" MATCHES "[c]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xC)
	elseif ("${_STM_CPU_SUFFIX}" MATCHES "[de]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xE)
	elseif ("${_STM_CPU_SUFFIX}" MATCHES "[fg]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xG)
	else()
		message(FATAL_ERROR "Unsupported CPU[${TARGET_CPU}]!")
	endif()
elseif ("${TARGET_CPU}" MATCHES "^stm32f([3])([0-9][0-9]).([468bcdefg])$")
	set(_STM_CPU_PREFIX "STM32F${CMAKE_MATCH_1}${CMAKE_MATCH_2}")
	set(_STM_CPU_SUFFIX ${CMAKE_MATCH_3})
	set(TARGET_CPU_TYPE_COMMON "STM32F${CMAKE_MATCH_1}xx")
	if ("${_STM_CPU_PREFIX}" MATCHES "STM32F3[12579]8")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xx)
	elseif ("${_STM_CPU_SUFFIX}" MATCHES "[68]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}x8)
	elseif ("${_STM_CPU_SUFFIX}" MATCHES "[bc]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xC)
	elseif ("${_STM_CPU_SUFFIX}" MATCHES "[de]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xE)
	elseif ("${_STM_CPU_SUFFIX}" MATCHES "[fg]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xG)
	else()
		message(FATAL_ERROR "Unsupported CPU[${TARGET_CPU}]!")
	endif()
elseif ("${TARGET_CPU}" MATCHES "^stm32f([2][0-9][0-9]).([468bcdefg])$")
	set(TARGET_CPU_TYPE_DEF "STM32F${CMAKE_MATCH_1}xx")
	set(TARGET_CPU_TYPE_COMMON "STM32F2xx")
elseif ("${TARGET_CPU}" MATCHES "^stm32f([4][0-9][0-9]).([0-9a-z])$")
	set(_STM_CPU_PREFIX "STM32F${CMAKE_MATCH_1}")
	set(_STM_CPU_SUFFIX ${CMAKE_MATCH_2})
	if ("${_STM_CPU_PREFIX}" STREQUAL "STM32F401")
		if ("${_STM_CPU_SUFFIX}" MATCHES "[bc]")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xC)
		elseif ("${_STM_CPU_SUFFIX}" MATCHES "[de]")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xE)
		else()
			set(TARGET_CPU_TYPE_DEF "STM32F${CMAKE_MATCH_1}xx")
		endif()
	elseif ("${_STM_CPU_PREFIX}" STREQUAL "STM32F410")
		if ("${TARGET_CPU}" MATCHES "^stm32f410t.*")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}Tx)
		elseif ("${TARGET_CPU}" MATCHES "^stm32f410c.*")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}Cx)
		elseif ("${TARGET_CPU}" MATCHES "^stm32f410r.*")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}Rx)
		else()
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xx)
		endif()
	elseif ("${_STM_CPU_PREFIX}" STREQUAL "STM32F411")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xE)
	elseif ("${_STM_CPU_PREFIX}" STREQUAL "STM32F412")
		if ("${TARGET_CPU}" MATCHES "^stm32f412c.*")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}Cx)
		elseif ("${TARGET_CPU}" MATCHES "^stm32f412z.*")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}Zx)
		elseif ("${TARGET_CPU}" MATCHES "^stm32f412v.*")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}Vx)
		elseif ("${TARGET_CPU}" MATCHES "^stm32f412r.*")
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}Rx)
		else()
			set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xx)
		endif()
	else()
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xx)
	endif()
	set(TARGET_CPU_TYPE_COMMON "STM32F4xx")
elseif ("${TARGET_CPU}" MATCHES "^stm32f([7][0-9][0-9]).([0-9a-z])$")
	set(TARGET_CPU_TYPE_DEF "STM32F${CMAKE_MATCH_1}xx")
	set(TARGET_CPU_TYPE_COMMON "STM32F7xx")
elseif ("${TARGET_CPU}" MATCHES "^stm32h([5][0-9][0-9]).([0-9a-z])$")
	set(TARGET_CPU_TYPE_DEF "STM32H${CMAKE_MATCH_1}xx")
	set(TARGET_CPU_TYPE_COMMON "STM32H5xx")
elseif ("${TARGET_CPU}" MATCHES "^stm32h([7][0-9ab][0-9]).([0-9a-z])$")
	set(_STM_CPU_PREFIX "STM32H${CMAKE_MATCH_1}")
	string(TOUPPER ${_STM_CPU_PREFIX} _STM_CPU_PREFIX)
	set(_STM_CPU_SUFFIX ${CMAKE_MATCH_2})
	if ("${TARGET_CPU}" MATCHES "^stm32h74[57].g.*")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xG)
	elseif ("${TARGET_CPU}" MATCHES "^stm32h7....q.*")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xxQ)
	else()
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xx)
	endif()
	set(TARGET_CPU_TYPE_COMMON "STM32H7xx")
elseif ("${TARGET_CPU}" MATCHES "^stm32g([0][0-9][0-9]).([0-9a-z])$")
	set(TARGET_CPU_TYPE_DEF "STM32G${CMAKE_MATCH_1}xx")
	set(TARGET_CPU_TYPE_COMMON "STM32G0xx")
elseif ("${TARGET_CPU}" MATCHES "^stm32g([4][0-9][0-9]).([0-9a-z])$")
	set(TARGET_CPU_TYPE_COMMON "STM32G${CMAKE_MATCH_1}xx")
	set(TARGET_CPU_TYPE_DEF "STM32G${CMAKE_MATCH_1}xx")
elseif ("${TARGET_CPU}" MATCHES "^stm32gbk1cb$")
	set(TARGET_CPU_TYPE_COMMON "STM32G4xx")
	set(TARGET_CPU_TYPE_DEF "STM32GBK1CB")
elseif ("${TARGET_CPU}" MATCHES "^stm32l([0])(10).([468bcdefg])$")
	set(_STM_CPU_PREFIX "STM32L${CMAKE_MATCH_1}${CMAKE_MATCH_2}")
	set(_STM_CPU_SUFFIX ${CMAKE_MATCH_3})
	set(TARGET_CPU_TYPE_COMMON "STM32L${CMAKE_MATCH_1}xx")
	if ("${_STM_CPU_SUFFIX}" MATCHES "[4]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}x4)
	elseif ("${_STM_CPU_SUFFIX}" MATCHES "[6]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}x6)
	elseif ("${_STM_CPU_SUFFIX}" MATCHES "[8]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}x8)
	elseif ("${_STM_CPU_SUFFIX}" MATCHES "[b]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xB)
	else()
		message(FATAL_ERROR "Unsupported CPU[${TARGET_CPU}]!")
	endif()
elseif ("${TARGET_CPU}" MATCHES "^stm32l([0][0-9][0-9]).([0-9a-z])$")
	set(TARGET_CPU_TYPE_COMMON "STM32L${CMAKE_MATCH_1}xx")
	set(TARGET_CPU_TYPE_DEF "STM32L${CMAKE_MATCH_1}xx")
elseif ("${TARGET_CPU}" MATCHES "^stm32l([1])([0-9][0-9]).([468bcdefg])([ax]?)?$")
	set(_STM_CPU_PREFIX "STM32L${CMAKE_MATCH_1}${CMAKE_MATCH_2}")
	set(_STM_CPU_SUFFIX ${CMAKE_MATCH_3})
	set(_STM_CPU_SUFFIX2 ${CMAKE_MATCH_4})
	set(TARGET_CPU_TYPE_COMMON "STM32L${CMAKE_MATCH_1}xx")
	if ("${_STM_CPU_SUFFIX}" MATCHES "[68b]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xB${_STM_CPU_SUFFIX2})
	elseif ("${_STM_CPU_SUFFIX}" MATCHES "[c]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xC${_STM_CPU_SUFFIX2})
	elseif ("${_STM_CPU_SUFFIX}" MATCHES "[d]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xD${_STM_CPU_SUFFIX2})
	elseif ("${_STM_CPU_SUFFIX}" MATCHES "[e]")
		set(TARGET_CPU_TYPE_DEF ${_STM_CPU_PREFIX}xE${_STM_CPU_SUFFIX2})
	else()
		message(FATAL_ERROR "Unsupported CPU[${TARGET_CPU}]!")
	endif()
elseif ("${TARGET_CPU}" MATCHES "^stm32l([45][0-9rs][0-9]).([0-9a-z])$")
	set(_STM_CPU_PREFIX "STM32L${CMAKE_MATCH_1}")
	string(TOUPPER ${_STM_CPU_PREFIX} _STM_CPU_PREFIX)
	set(TARGET_CPU_TYPE_COMMON "${_STM_CPU_PREFIX}xx")
	set(TARGET_CPU_TYPE_DEF "${_STM_CPU_PREFIX}xx")
else()
	message(FATAL_ERROR "Unsupported device [${TARGET_CPU}]!")
endif()

set(TARGET_CPU_TYPE cortex-m0)
set(TARGET_ARM_MATH_TYPE ARM_MATH_CM0)
if ("${TARGET_CPU}" MATCHES "^stm32(f[12]|l[1]).*")
	set(TARGET_CPU_TYPE cortex-m3)
	set(TARGET_ARM_MATH_TYPE ARM_MATH_CM3)
elseif("${TARGET_CPU}" MATCHES "^stm32(f[34]|g[4]|l[4]).*")
	set(TARGET_CPU_TYPE cortex-m4)
	set(TARGET_ARM_MATH_TYPE ARM_MATH_CM4)
elseif("${TARGET_CPU}" MATCHES "^stm32(f[7]|h[7]).*")
	set(TARGET_CPU_TYPE cortex-m7)
	set(TARGET_ARM_MATH_TYPE ARM_MATH_CM7)
elseif("${TARGET_CPU}" MATCHES "^stm32(h[5]|l[5]|u[5]).*")
	set(TARGET_CPU_TYPE cortex-m33)
	set(TARGET_ARM_MATH_TYPE ARM_MATH_ARMV8MBL)
elseif("${TARGET_CPU}" MATCHES "^stm32(g[0]|c[0]|l[0]).*")
	set(TARGET_CPU_TYPE cortex-m0plus)
	set(TARGET_ARM_MATH_TYPE ARM_MATH_CM0PLUS)
elseif("${TARGET_CPU}" MATCHES "^stm32mp1.*")
	set(TARGET_CPU_TYPE cortex-a7)
endif()
message("--      Device : " ${TARGET_CPU})
message("--         CPU : " ${TARGET_CPU_TYPE})


if(CLANG)
	add_compile_options(--target=arm-none-eabi -Wno-unused-command-line-argument)
	add_link_options(--target=arm-none-eabi -Wno-unused-command-line-argument)
endif()

add_compile_definitions(${TARGET_CPU_TYPE_COMMON} ${TARGET_CPU_TYPE_DEF})
add_compile_options(
	-mthumb
	-mcpu=${TARGET_CPU_TYPE}
	-Wfatal-errors
	-fno-exceptions
	-fno-unwind-tables
	-fno-stack-check
)

set(CMAKE_CXX_FLAGS	"${CMAKE_CXX_FLAGS} -fno-rtti -fno-exceptions -fuse-cxa-atexit")
add_link_options(
	-mthumb
	-mcpu=${TARGET_CPU_TYPE}
	-Wl,--print-memory-usage
	-Wl,--warn-once
	-L"$ENV{SDK_LIB}"
)

message("--       Specs : " ${TARGET_SPECS})
if (CLANG)
	if ("${TARGET_SPECS}" STREQUAL "nano")
		# alias as pico
		add_link_options(-lcrt0-minimal -ldummyhost)
	elseif ("${TARGET_SPECS}" STREQUAL "nosys")
		add_link_options(-lcrt0 -ldummyhost)
	elseif ("${TARGET_SPECS}" STREQUAL "rdimon")
		# should invoke initialise_monitor_handles
		add_link_options(-lcrt0-semihost -lsemihost)
	elseif ("${TARGET_SPECS}" STREQUAL "rdimon-v2m")
		# should invoke initialise_monitor_handles
		add_link_options(-lcrt0-semihost -lsemihost)
	elseif ("${TARGET_SPECS}" STREQUAL "none")
	else()
		message("${_WARNING_PREFIX} : Unsupported Specs " ${TARGET_SPECS})
	endif()
else()
	if ("${TARGET_SPECS}" STREQUAL "nano")
		add_link_options(--specs=nano.specs)
		add_link_options(-lnosys --specs=nosys.specs)
	elseif ("${TARGET_SPECS}" STREQUAL "nosys")
		add_link_options(-lnosys --specs=nosys.specs)
	elseif ("${TARGET_SPECS}" STREQUAL "rdimon")
		# should invoke initialise_monitor_handles
		add_link_options(-lrdimon --specs=rdimon.specs)
	elseif ("${TARGET_SPECS}" STREQUAL "rdimon-v2m")
		# should invoke initialise_monitor_handles
		add_link_options(-lrdimon-v2m --specs=rdimon-v2m.specs)
	elseif ("${TARGET_SPECS}" STREQUAL "none")
	else()
		message("${_WARNING_PREFIX} : Unsupported Specs " ${TARGET_SPECS})
	endif()
endif()

if (_IS_CUBEMX_PROJECT)
	include_directories(${_CUBEMX_INCLUDE_DIRS})
else()
	include_directories(${CMAKE_CURRENT_SOURCE_DIR}/inc $ENV{SDK_INCLUDE})
endif()

if ("${TARGET_CPU_FLOAT_ABI}" MATCHES "soft|softfp|hard")
	add_compile_options(-mfloat-abi=${TARGET_CPU_FLOAT_ABI})
	add_link_options(-mfloat-abi=${TARGET_CPU_FLOAT_ABI})
	message("--   Float ABI : " ${TARGET_CPU_FLOAT_ABI})
elseif(NOT "${TARGET_CPU_FLOAT_ABI}" STREQUAL "none")
	message("${_WARNING_PREFIX} : Invalid Float ABI ${TARGET_CPU_FLOAT_ABI}")
endif()

if ("${TARGET_HSE_VALUE}" STREQUAL "auto")
	message("--   HSE_VALUE : auto")
elseif("${TARGET_HSE_VALUE}" MATCHES "^[0-9]+$")
	add_compile_definitions(HSE_VALUE=${TARGET_HSE_VALUE})
	message("--   HSE_VALUE : " ${TARGET_HSE_VALUE})
else()
	message("${_WARNING_PREFIX} : Invalid HSE_VALUE ${TARGET_HSE_VALUE}")
endif()

if ("${TARGET_HSI_VALUE}" STREQUAL "auto")
	message("--   HSI_VALUE : auto")
elseif("${TARGET_HSI_VALUE}" MATCHES "^[0-9]+$")
	add_compile_definitions(HSI_VALUE=${TARGET_HSI_VALUE})
	message("--   HSI_VALUE : " ${TARGET_HSI_VALUE})
else()
	message("${_WARNING_PREFIX} : Invalid HSI_VALUE ${TARGET_HSI_VALUE}")
endif()

set(STM32CUBE_SDK)

if (_IS_CUBEMX_PROJECT)
	list(FILTER _CUBEMX_DRIVERS_SOURCES EXCLUDE REGEX .*_template\.c)
	list(FILTER _CUBEMX_DRIVERS_SOURCES EXCLUDE REGEX ".*/(NN|Core_A|Core/Template|RTOS2|DSP/(DSP_Lib_TestSuite|Examples)|Source/Templates)/.*")
	set(_CUBEMX_DRIVERS_DSP_SOURCES ${_CUBEMX_DRIVERS_SOURCES})
	list(FILTER _CUBEMX_DRIVERS_DSP_SOURCES INCLUDE REGEX ".*/DSP/.*")
	list(FILTER _CUBEMX_DRIVERS_SOURCES EXCLUDE REGEX ".*/DSP/.*")
	
	add_library(cubemx-driver STATIC
		${_CUBEMX_DRIVERS_SOURCES}
	)
	list(APPEND STM32CUBE_SDK cubemx-driver)

	if (_CUBEMX_FATFS_SOURCES)
		add_library(cubemx-fatfs STATIC
			${_CUBEMX_FATFS_SOURCES}
		)
		list(APPEND STM32CUBE_SDK cubemx-fatfs)
	endif()

	if (_CUBEMX_USB_DEVICE_SOURCES)
		add_library(cubemx-usb-device STATIC
			${_CUBEMX_USB_DEVICE_SOURCES}
		)
		list(APPEND STM32CUBE_SDK cubemx-usb-device)
	endif()

	if (_CUBEMX_MIDDLEWARES_SOURCES)
		add_library(cubemx-middlewares STATIC
			${_CUBEMX_MIDDLEWARES_SOURCES}
		)
		list(APPEND STM32CUBE_SDK cubemx-middlewares)
	endif()

	if (ENABLE_DSP_LIB)
		set_source_files_properties(
			${_CUBEMX_DRIVERS_DSP_SOURCES}
			PROPERTIES COMPILE_DEFINITIONS "${TARGET_ARM_MATH_TYPE}"
		)
		add_library(cubemx-driver-dsp STATIC
			${_CUBEMX_DRIVERS_DSP_SOURCES}
		)
		list(APPEND STM32CUBE_SDK cubemx-driver-dsp)
		message("--     DSP_LIB : enabled")
	else()
		message("--     DSP_LIB : disabled")
	endif()
else()
	if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/sdk/${TARGET_CPU_TYPE_DEF}_FLASH.ld)
		if (EXISTS $ENV{SDK_CMSIS_DEVICES_ST_SRC}/Templates/gcc/linker/${TARGET_CPU_TYPE_DEF}_FLASH.ld)
			file(COPY_FILE
				$ENV{SDK_CMSIS_DEVICES_ST_SRC}/Templates/gcc/linker/${TARGET_CPU_TYPE_DEF}_FLASH.ld
				${CMAKE_CURRENT_SOURCE_DIR}/sdk/${TARGET_CPU_TYPE_DEF}_FLASH.ld
			)
		endif()
	endif()
	# copy system_xxxx.c
	if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/sdk/system_${TARGET_CPU_TYPE_COMMON}.c)
		file(COPY_FILE
			$ENV{SDK_CMSIS_DEVICES_ST_SRC}/Templates/system_${TARGET_CPU_TYPE_COMMON}.c
			${CMAKE_CURRENT_SOURCE_DIR}/sdk/system_${TARGET_CPU_TYPE_COMMON}.c
		)
	endif()

	# copy startup_xxxx.s
	if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/sdk/startup_${TARGET_CPU_TYPE_DEF}.s)
		file(COPY_FILE
			$ENV{SDK_CMSIS_DEVICES_ST_SRC}/Templates/GCC/startup_${TARGET_CPU_TYPE_DEF}.s
			${CMAKE_CURRENT_SOURCE_DIR}/sdk/startup_${TARGET_CPU_TYPE_DEF}.s
		)
	endif()

	if (ENABLE_HAL_DRIVER)
		message("--  HAL Driver : enabled")
		file(GLOB_RECURSE HAL_DRIVER_SOURCES
			LIST_DIRECTORIES false
			$ENV{SDK_HAL_DRIVER_SRC}/*_hal_*.c
			$ENV{SDK_HAL_DRIVER_SRC}/*_hal.c
		)
		list(TRANSFORM HAL_DRIVER_SOURCES REPLACE "\\\\" "/")

		# Prepare HAL templates
		set(HAL_DRIVER_SOURCES_TEMPLATES ${HAL_DRIVER_SOURCES})
		list(FILTER HAL_DRIVER_SOURCES_TEMPLATES INCLUDE REGEX .*_template\.c)
		list(FILTER HAL_DRIVER_SOURCES EXCLUDE REGEX .*_template\.c)

		if (HAL_DRIVER_SOURCES_TEMPLATES)
			if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/hal)
				file(MAKE_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}/hal)
				foreach(TEMPLATE_ITEM IN LISTS HAL_DRIVER_SOURCES_TEMPLATES)
					string(REGEX REPLACE ".*/(.+)_template\\.c" "\\1.c" LOCAL_FILE_NAME "${TEMPLATE_ITEM}")
					file(COPY_FILE
						${TEMPLATE_ITEM}
						${CMAKE_CURRENT_SOURCE_DIR}/hal/${LOCAL_FILE_NAME}-template
					)
				endforeach()
			endif()
		endif()

		add_library(hal-driver STATIC
			${HAL_DRIVER_SOURCES}
		)

		list(APPEND STM32CUBE_SDK hal-driver)

		string(TOLOWER "${TARGET_CPU_TYPE_COMMON}" _HAL_CONF_FILE_PREFIX)
		if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/inc/${_HAL_CONF_FILE_PREFIX}_hal_conf.h)
			if (NOT EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/inc)
				file(MAKE_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}/inc)
			endif()
			file(COPY_FILE
				$ENV{SDK_HAL_DRIVER_SRC}/../Inc/${_HAL_CONF_FILE_PREFIX}_hal_conf_template.h
				${CMAKE_CURRENT_SOURCE_DIR}/inc/${_HAL_CONF_FILE_PREFIX}_hal_conf.h
			)
		endif()
	else()
		message("--  HAL Driver : disabled")
	endif()

	if (ENABLE_LL_DRIVER)
		message("--   LL Driver : enabled")
		file(GLOB_RECURSE LL_DRIVER_SOURCES
			LIST_DIRECTORIES false
			$ENV{SDK_HAL_DRIVER_SRC}/*_ll_*.c
		)
		list(TRANSFORM LL_DRIVER_SOURCES REPLACE "\\\\" "/")
		add_library(ll-driver STATIC
			${LL_DRIVER_SOURCES}
		)
		list(APPEND STM32CUBE_SDK ll-driver)
	else()
		message("--   LL Driver : disabled")
	endif()

	if (ENABLE_DSP_LIB)
		set(CMSIS_DSP_ROOT $ENV{SDK_CMSIS_DEVICES_ST_SRC}/../../../../DSP)
		file(GLOB_RECURSE CMSIS_DSP_SOURCES
			LIST_DIRECTORIES false
			${CMSIS_DSP_ROOT}/Source/*.c
		)
		list(TRANSFORM CMSIS_DSP_SOURCES REPLACE "\\\\" "/")

		add_library(cmsis-dsp STATIC
			${CMSIS_DSP_SOURCES}
		)
		set_source_files_properties(
			${CMSIS_DSP_SOURCES}
			PROPERTIES INCLUDE_DIRECTORIES "${CMSIS_DSP_ROOT}/PrivateInclude"
		)

		list(APPEND STM32CUBE_SDK cmsis-dsp)
		message("--     DSP_LIB : enabled")
	else()
		message("--     DSP_LIB : disabled")
	endif()

	if (ENABLE_NN_LIB)
		set(CMSIS_NN_ROOT $ENV{SDK_CMSIS_DEVICES_ST_SRC}/../../../../NN)
		file(GLOB_RECURSE CMSIS_NN_SOURCES
			LIST_DIRECTORIES false
			${CMSIS_NN_ROOT}/Source/*.c
		)
		list(TRANSFORM CMSIS_NN_SOURCES REPLACE "\\\\" "/")
		include_directories(${CMSIS_NN_ROOT}/include)
		add_library(cmsis-nn STATIC
			${CMSIS_NN_SOURCES}
		)
		# set_source_files_properties(
		# 	${CMSIS_NN_SOURCES}
		# 	PROPERTIES INCLUDE_DIRECTORIES "${CMSIS_NN_ROOT}/"
		# )

		list(APPEND STM32CUBE_SDK cmsis-nn)
		message("--      NN_LIB : enabled")
	else()
		message("--      NN_LIB : disabled")
	endif()
endif() # NOT _IS_CUBEMX_PROJECT

# find ld script
if (_IS_CUBEMX_PROJECT)
	add_link_options(-Wl,-T,${CMAKE_CURRENT_SOURCE_DIR}/${_CUBEMX_LINK_SCRIPT})
	message("--   LD Script : ${_CUBEMX_LINK_SCRIPT}")
elseif (EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/SDK/${TARGET_CPU_TYPE_DEF}_FLASH.ld)
	add_link_options(-Wl,-T,${CMAKE_CURRENT_SOURCE_DIR}/SDK/${TARGET_CPU_TYPE_DEF}_FLASH.ld)
	message("--   LD Script : sdk/${TARGET_CPU_TYPE_DEF}_FLASH.ld")
elseif(EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/SDK/${TARGET_CPU_TYPE_COMMON}_FLASH.ld)
	add_link_options(-Wl,-T,${CMAKE_CURRENT_SOURCE_DIR}/SDK/${TARGET_CPU_TYPE_COMMON}_FLASH.ld)
	message("--   LD Script : sdk/${TARGET_CPU_TYPE_COMMON}_FLASH.ld")
else()
	set(_LINK_FILE_PREFIX ${TARGET_CPU})
	string(TOUPPER "${_LINK_FILE_PREFIX}" _LINK_FILE_PREFIX)
	string(REGEX REPLACE "^STM32(....).*" "STM32\\1xx" _LINK_FILE_PREFIX "${_LINK_FILE_PREFIX}")
	if(EXISTS ${CMAKE_CURRENT_SOURCE_DIR}/SDK/${_LINK_FILE_PREFIX}_FLASH.ld)
		add_link_options(-Wl,-T,${CMAKE_CURRENT_SOURCE_DIR}/SDK/${_LINK_FILE_PREFIX}_FLASH.ld)
		message("--   LD Script : sdk/${_LINK_FILE_PREFIX}_FLASH.ld")
	else()
		message("${_WARNING_PREFIX} : LD script not found!")
	endif()
endif()

include(${CMAKE_CURRENT_LIST_DIR}/segger-rtt.cmake)

if (_IS_CUBEMX_PROJECT)
	# _CUBEMX_CORE_SOURCES
	# _CUBEMX_STARTUP_SOURCE
	list(APPEND SOURCE_FILES
		${SEGGER_RTT_SOURCES}
		${_CUBEMX_STARTUP_SOURCE}
		${_CUBEMX_CORE_SOURCES}
	)
else()
	file(GLOB_RECURSE SOURCE_FILES
		LIST_DIRECTORIES false
		${SEGGER_RTT_SOURCES}
		sdk/*.cxx
		sdk/*.cpp
		sdk/*.cc
		sdk/*.c
		sdk/*.s
		src/*.cxx
		src/*.cpp
		src/*.cc
		src/*.c
		src/*.s
		hal/*.s
		hal/*.c
		hal/*.cxx
		hal/*.cpp
		hal/*.cc
	)
endif()
