﻿local CMAKE_LIST_TEMPLATE_FIX_COMPILER_ARCH_ID = [[
# fix CMAKE_C_COMPILER_ARCHITECTURE_ID for gcc/clang
if (NOT CMAKE_C_COMPILER_ARCHITECTURE_ID)
	if(CMAKE_C_COMPILER_ID MATCHES "GNU|Clang")
		if (CMAKE_C_COMPILER_PREDEFINES_COMMAND)
			set(_COMPILER_PATH ${CMAKE_C_COMPILER_PREDEFINES_COMMAND})
		elseif (CMAKE_CXX_COMPILER_PREDEFINES_COMMAND)
			set(_COMPILER_PATH ${CMAKE_CXX_COMPILER_PREDEFINES_COMMAND})
		endif()
		if ("-m32" IN_LIST _COMPILER_PATH)
			set(CMAKE_C_COMPILER_ARCHITECTURE_ID x86)
		elseif (_COMPILER_PATH MATCHES ".*[/\\\\]([^-]+)-(.+)-(g\\+\\+|gcc|clang|clang\\+\\+)\\..*")
			set(CMAKE_C_COMPILER_ARCHITECTURE_ID ${CMAKE_MATCH_1})
		endif()

		if (NOT CMAKE_C_COMPILER_ARCHITECTURE_ID)
			execute_process(
				COMMAND ${_COMPILER_PATH} -v
				OUTPUT_VARIABLE  _COMPILER_V_OUTPUT
				ERROR_VARIABLE  _COMPILER_V_OUTPUT
			)
			string(REGEX MATCH "Target:[ \\t]*([^ \t\n]*)" GCC_TARGET "${_COMPILER_V_OUTPUT}")
			string(REGEX MATCH "([^-]*)-([^-]*)-(.*)" CMAKE_C_COMPILER_ARCHITECTURE_ID "${CMAKE_MATCH_1}")
			set(CMAKE_C_COMPILER_ARCHITECTURE_ID ${CMAKE_MATCH_1})
			unset(_COMPILER_V_OUTPUT)
		endif()
		unset(_COMPILER_PATH)
	endif()
endif()
string(REGEX REPLACE "[xX]86_64|X64" "x64" CMAKE_C_COMPILER_ARCHITECTURE_ID ${CMAKE_C_COMPILER_ARCHITECTURE_ID})
string(REGEX REPLACE "[iI][1-7]86|X86" "x86" CMAKE_C_COMPILER_ARCHITECTURE_ID ${CMAKE_C_COMPILER_ARCHITECTURE_ID})]];
local CMAKE_LIST_TEMPLATE_MSVC_OPTIONS = [[
if (MSVC)
	option(MSVC_USE_STATIC_CRT "Build target(s) with static crt library" OFF)
	string(TOUPPER ${CMAKE_BUILD_TYPE} _MSVC_BUILD_CONFIG)
	set(_EDX_ESC "]].."\x1b["..[[")
	if (MSVC_USE_STATIC_CRT)
		message("-- Using ${_EDX_ESC}34mSTATIC${_EDX_ESC}0m linked CRT (/MT)")
		string(REPLACE "/MD" "/MT" CMAKE_C_FLAGS_${_MSVC_BUILD_CONFIG} ${CMAKE_C_FLAGS_${_MSVC_BUILD_CONFIG}})
		string(REPLACE "/MD" "/MT" CMAKE_CXX_FLAGS_${_MSVC_BUILD_CONFIG} ${CMAKE_CXX_FLAGS_${_MSVC_BUILD_CONFIG}})
	else()
		message("-- Using ${_EDX_ESC}34mDYNAMIC${_EDX_ESC}0m linked CRT (/MD)")
		string(REPLACE "/MT" "/MD" CMAKE_C_FLAGS_${_MSVC_BUILD_CONFIG} ${CMAKE_C_FLAGS_${_MSVC_BUILD_CONFIG}})
		string(REPLACE "/MT" "/MD" CMAKE_CXX_FLAGS_${_MSVC_BUILD_CONFIG} ${CMAKE_CXX_FLAGS_${_MSVC_BUILD_CONFIG}})
	endif()
else()
	set(MSVC_USE_STATIC_CRT)
endif()]];

--[[
info = {
	["type"] = "exec"; -- "shared", "static"
	["project"] = "project-name";
	["name"] = "target-name";
	["nodep"] = false; -- true/false
	["embeded"] = false; -- false/true/"pico"
	["target_device"] = "stm32f103c8";
	["extra"] = "";
}
]]
function make_cmake_list(info)
	local cmake_list_template = {"cmake_minimum_required(VERSION 3.8)"};
	table.insert(cmake_list_template, [[]]);
	if info.embeded then
		if info.embeded == "pico" then
			table.insert(cmake_list_template, [[include($ENV{PICO_SDK_PATH}/pico_sdk_init.cmake)]]);
			table.insert(cmake_list_template, [[]]);
		else
			table.insert(cmake_list_template, ([[set(TARGET_CPU %s)]]):format(info.target_device));
			table.insert(cmake_list_template, [[include($ENV{_EDX_HOME}/cmake/embeded-support.cmake)]]);
			table.insert(cmake_list_template, [[]]);
		end;
	end;
	table.insert(cmake_list_template, ([[project("%s")]]):format(info.project));
	table.insert(cmake_list_template, [[]]);
	if info.embeded == "pico" then
		table.insert(cmake_list_template, [[pico_sdk_init()]]);
		table.insert(cmake_list_template, [[]]);
	end;
	table.insert(cmake_list_template, ([[set(TARGET_NAME %s)]]):format(info.name));
	table.insert(cmake_list_template, [[]]);
	if info.embeded == "pico" then
		table.insert(cmake_list_template, [[set(CMAKE_C_STANDARD 17)]]);
		table.insert(cmake_list_template, [[set(CMAKE_CXX_STANDARD 17)]]);
	else
		table.insert(cmake_list_template, [[set(CMAKE_C_STANDARD 11)]]);
		table.insert(cmake_list_template, [[set(CMAKE_CXX_STANDARD 14)]]);
	end
	table.insert(cmake_list_template, [[]]);
	table.insert(cmake_list_template, CMAKE_LIST_TEMPLATE_FIX_COMPILER_ARCH_ID);
	table.insert(cmake_list_template, [[]]);
	if not info.embeded then
		table.insert(cmake_list_template, CMAKE_LIST_TEMPLATE_MSVC_OPTIONS);
		table.insert(cmake_list_template, [[]]);
	end;
	table.insert(cmake_list_template, [[# include_directories(<HEADER_PATH>)]]);
	table.insert(cmake_list_template, [[]]);
	if info.extra then
		table.insert(cmake_list_template, info.extra);
		table.insert(cmake_list_template, [[]]);
	end;
	table.insert(cmake_list_template, [[# add_compile_options(<COMPILER_FLAGS>)]]);
    table.insert(cmake_list_template, [[]]);

    if info.embeded == "pico" then
        table.insert(cmake_list_template, [[aux_source_directory(./src SOURCE_FILES)]]);
    elseif info.embeded then
    else
        table.insert(cmake_list_template, [[aux_source_directory(./ SOURCE_FILES)]]);
    end;

	if info["type"] == "exec" then
		table.insert(cmake_list_template, [[add_executable(${TARGET_NAME} ${SOURCE_FILES})]]);
	elseif info["type"] == "shared" then
		table.insert(cmake_list_template, [[add_library(${TARGET_NAME} SHARED ${SOURCE_FILES})]]);
	else
		table.insert(cmake_list_template, [[add_library(${TARGET_NAME} STATIC ${SOURCE_FILES})]]);
	end;
	table.insert(cmake_list_template, [[]]);
	table.insert(cmake_list_template, [[# target_precompile_headers(${TARGET_NAME} PRIVATE pch_header.h)]]);
	table.insert(cmake_list_template, [[]]);
	if info.target_link_libs then
		table.insert(cmake_list_template, [[target_link_libraries(${TARGET_NAME}]]);
		if type(info.target_link_libs) == "table" then
			for idx, lib in ipairs(info.target_link_libs) do
				table.insert(cmake_list_template, [[	]]..lib);
			end;
		else
			table.insert(cmake_list_template, [[	]]..info.target_link_libs);
		end;
		table.insert(cmake_list_template, [[)]]);
	else
		table.insert(cmake_list_template, [[# target_link_libraries(${TARGET_NAME}]]);
		table.insert(cmake_list_template, [[#	<LIB_NAME>]]);
		table.insert(cmake_list_template, [[# )]]);
	end;
	table.insert(cmake_list_template, [[]]);
	table.insert(cmake_list_template, [[set(CMAKE_INSTALL_PREFIX ${CMAKE_CURRENT_SOURCE_DIR}/output/${CMAKE_C_COMPILER_ID}-${CMAKE_C_COMPILER_ARCHITECTURE_ID}-${CMAKE_BUILD_TYPE})]]);
	table.insert(cmake_list_template, [[]]);
	table.insert(cmake_list_template, [[install(TARGETS ${TARGET_NAME}]]);
	if not (info.embeded or info.nodep) then
		table.insert(cmake_list_template, [[	RUNTIME_DEPENDENCIES]]);
		table.insert(cmake_list_template, [[	PRE_EXCLUDE_REGEXES "system32/|((api-ms-).*\\.dll)|/usr/|/lib/|/lib64/"]]);
		table.insert(cmake_list_template, [[	POST_EXCLUDE_REGEXES "system32/|((api-ms-).*\\.dll)|/usr/|/lib/|/lib64/"]]);
	end;
	table.insert(cmake_list_template, [[	RUNTIME DESTINATION bin]]);
	table.insert(cmake_list_template, [[	LIBRARY DESTINATION lib]]);
	table.insert(cmake_list_template, [[	ARCHIVE DESTINATION lib)]]);
	table.insert(cmake_list_template, [[]]);
	table.insert(cmake_list_template, [[# install(FILES ${SOURCE_FILES}]]);
	table.insert(cmake_list_template, [[#	DESTINATION src)]]);
	table.insert(cmake_list_template, [[]]);

	return table.concat(cmake_list_template, "\n");
end;

local function make_device_ld_script(dev_info)
	local heap_size = 0x200;
	local stack_size = 0x400;
	local stackseal_size = 0;
	local script_info = {};
	if arm_cmse then
		stackseal_size = 8;
	else
		stackseal_size = 0;
	end;
	local ram_idx = -1;
	local rom_idx = -1;
	local mem_info = dev_info[3];
	table.insert(script_info, [[ENTRY(Reset_Handler)]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[MEMORY]]);
	table.insert(script_info, [[{]]);
	local ram_end = 0;
	for idx, info in ipairs(mem_info) do
		local begin = info[1];
		local size = info[2];
		local flags = info[3];
		local t = flags&0xF;
		local t_name = nil;
		local midx = 0;
		local access_mode = "rx"
		if t == 0 then
			t_name = "RAM";
			ram_idx = ram_idx + 1;
			access_mode = "xrw"
			if ram_idx == 0 then
				midx = "";
			else
				midx = ram_idx;
			end;
			ram_end = begin + size - 1;
		elseif t == 1 then
			t_name = "FLASH";
			rom_idx = rom_idx + 1;
			access_mode = "rx"
			if rom_idx == 0 then
				midx = "";
			else
				midx = rom_idx;
			end;
		end;
		if t_name then
			if size>=1024 and (size&1023) == 0 then
				table.insert(script_info, string.format([[  %s%s  (%s)  : ORIGIN = 0x%08X, LENGTH = %uK]], t_name, midx, access_mode, begin, size/1024));
			else
				table.insert(script_info, string.format([[  %s%s  (%s)  : ORIGIN = 0x%08X, LENGTH = 0x%08X]], t_name, midx, access_mode, begin, size));
			end;
		end;

	end;
	table.insert(script_info, [[}]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, string.format([[_estack = 0x%08X;]], ram_end));
	table.insert(script_info, string.format([[__HEAP_SIZE = 0x%08X;]], heap_size));
	table.insert(script_info, string.format([[__STACK_SIZE = 0x%08X;]],  stack_size));
	table.insert(script_info, string.format([[__STACKSEAL_SIZE = 0x%08X;]],  stackseal_size));
	table.insert(script_info, [[]]);
	table.insert(script_info, [[SECTIONS]]);
	table.insert(script_info, [[{]]);
	table.insert(script_info, [[  .isr_vector :]]);
	table.insert(script_info, [[  {]]);
	table.insert(script_info, [[    . = ALIGN(4);]]);
	table.insert(script_info, [[    KEEP(*(.isr_vector))]]);
	table.insert(script_info, [[    KEEP(*(.vectors))]]);
	table.insert(script_info, [[    . = ALIGN(4);]]);
	table.insert(script_info, [[  } >FLASH]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[  .text :]]);
	table.insert(script_info, [[  {]]);
	table.insert(script_info, [[	. = ALIGN(4);]]);
	table.insert(script_info, [[    *(.text)]]);
	table.insert(script_info, [[    *(.text*)]]);
	table.insert(script_info, [[    *(.glue_7)]]);
	table.insert(script_info, [[    *(.glue_7t)]]);
	table.insert(script_info, [[    *(.eh_frame)]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[    KEEP(*(.init))]]);
	table.insert(script_info, [[    KEEP(*(.fini))]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[    /* .ctors */]]);
	table.insert(script_info, [[    *crtbegin.o(.ctors)]]);
	table.insert(script_info, [[    *crtbegin?.o(.ctors)]]);
	table.insert(script_info, [[    *(EXCLUDE_FILE(*crtend?.o *crtend.o) .ctors)]]);
	table.insert(script_info, [[    *(SORT(.ctors.*))]]);
	table.insert(script_info, [[    *(.ctors)]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[    /* .dtors */]]);
	table.insert(script_info, [[    *crtbegin.o(.dtors)]]);
	table.insert(script_info, [[    *crtbegin?.o(.dtors)]]);
	table.insert(script_info, [[    *(EXCLUDE_FILE(*crtend?.o *crtend.o) .dtors)]]);
	table.insert(script_info, [[    *(SORT(.dtors.*))]]);
	table.insert(script_info, [[    *(.dtors)]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[    *(.rodata)]]);
	table.insert(script_info, [[    *(.rodata*)]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[    KEEP(*(.eh_frame*))]]);
	table.insert(script_info, [[    . = ALIGN(4);]]);
	table.insert(script_info, [[    _etext = .;]]);
	table.insert(script_info, [[  } > FLASH]]);
	table.insert(script_info, [[]]);
	if arm_cmse then
		table.insert(script_info, [[  .gnu.sgstubs :]]);
		table.insert(script_info, [[  {]]);
		table.insert(script_info, [[    . = ALIGN(32);]]);
		table.insert(script_info, [[  } > FLASH]]);
		table.insert(script_info, [[]]);
	end;
	table.insert(script_info, [[  .ARM.extab :]]);
	table.insert(script_info, [[  {]]);
	table.insert(script_info, [[    *(.ARM.extab* .gnu.linkonce.armextab.*)]]);
	table.insert(script_info, [[  } > FLASH]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[  __exidx_start = .;]]);
	table.insert(script_info, [[  .ARM.exidx :]]);
	table.insert(script_info, [[  {]]);
	table.insert(script_info, [[    *(.ARM.exidx* .gnu.linkonce.armexidx.*)]]);
	table.insert(script_info, [[  } > FLASH]]);
	table.insert(script_info, [[  __exidx_end = .;]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[  .preinit_array :]]);
	table.insert(script_info, [[  {]]);
	table.insert(script_info, [[    PROVIDE_HIDDEN (__preinit_array_start = .);]]);
	table.insert(script_info, [[    KEEP (*(.preinit_array*))]]);
	table.insert(script_info, [[    PROVIDE_HIDDEN (__preinit_array_end = .);]]);
	table.insert(script_info, [[  } >FLASH]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[  .init_array :]]);
	table.insert(script_info, [[  {]]);
	table.insert(script_info, [[    PROVIDE_HIDDEN (__init_array_start = .);]]);
	table.insert(script_info, [[    KEEP (*(SORT(.init_array.*)))]]);
	table.insert(script_info, [[    KEEP (*(.init_array*))]]);
	table.insert(script_info, [[    PROVIDE_HIDDEN (__init_array_end = .);]]);
	table.insert(script_info, [[  } >FLASH]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[  .fini_array :]]);
	table.insert(script_info, [[  {]]);
	table.insert(script_info, [[    PROVIDE_HIDDEN (__fini_array_start = .);]]);
	table.insert(script_info, [[    KEEP (*(SORT(.fini_array.*)))]]);
	table.insert(script_info, [[    KEEP (*(.fini_array*))]]);
	table.insert(script_info, [[    PROVIDE_HIDDEN (__fini_array_end = .);]]);
	table.insert(script_info, [[  } >FLASH]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[  _sidata = LOADADDR(.data);]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[  .data : ALIGN(4)]]);
	table.insert(script_info, [[  {]]);
	table.insert(script_info, [[    _sdata = .;]]);
	table.insert(script_info, [[    __data_start__ = _sdata;]]);
	table.insert(script_info, [[    *(.data.segger_rtt)]]);
	table.insert(script_info, [[    *(vtable)]]);
	table.insert(script_info, [[    *(.data)]]);
	table.insert(script_info, [[    *(.data.*)]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[    KEEP(*(.jcr*))]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[    . = ALIGN(4);]]);
	table.insert(script_info, [[    _edata = .;]]);
	table.insert(script_info, [[    __data_end__ = _edata;]]);
	table.insert(script_info, [[  } > RAM AT > FLASH]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[  .bss :]]);
	table.insert(script_info, [[  {]]);
	table.insert(script_info, [[    . = ALIGN(4);]]);
	table.insert(script_info, [[    _sbss = .;]]);
	table.insert(script_info, [[    __bss_start__ = _sbss;]]);
	table.insert(script_info, [[    *(.bss)]]);
	table.insert(script_info, [[    *(.bss.*)]]);
	table.insert(script_info, [[    *(COMMON)]]);
	table.insert(script_info, [[    . = ALIGN(4);]]);
	table.insert(script_info, [[    _ebss = .;]]);
	table.insert(script_info, [[    __bss_end__ = _ebss;]]);
	table.insert(script_info, [[  } > RAM AT > RAM]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[  .noinit (NOLOAD) :]]);
	table.insert(script_info, [[  {]]);
	table.insert(script_info, [[    . = ALIGN(4);]]);
	table.insert(script_info, [[    __noinit_start = .;]]);
	table.insert(script_info, [[    *(.noinit)]]);
	table.insert(script_info, [[    *(.noinit.*)]]);
	table.insert(script_info, [[    . = ALIGN(4);]]);
	table.insert(script_info, [[    __noinit_end = .;]]);
	table.insert(script_info, [[  } > RAM]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[  .heap (NOLOAD) :]]);
	table.insert(script_info, [[  {]]);
	table.insert(script_info, [[    . = ALIGN(8);]]);
	table.insert(script_info, [[    __end__ = .;]]);
	table.insert(script_info, [[    PROVIDE(end = .);]]);
	table.insert(script_info, [[    PROVIDE(__heap_start = .);]]);
	table.insert(script_info, [[    . = . + __HEAP_SIZE;]]);
	table.insert(script_info, [[    . = ALIGN(8);]]);
	table.insert(script_info, [[    PROVIDE(__heap_end = .);]]);
	table.insert(script_info, [[    __HeapLimit = .;]]);
	table.insert(script_info, [[  } > RAM]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[  .stack (NOLOAD) :]]);
	table.insert(script_info, [[  {]]);
	table.insert(script_info, [[    . = ALIGN(8);]]);
	table.insert(script_info, [[    __StackLimit = .;]]);
	table.insert(script_info, [[    . = . + __STACK_SIZE;]]);
	table.insert(script_info, [[    . = ALIGN(8);]]);
	table.insert(script_info, [[    __StackTop = .;]]);
	table.insert(script_info, [[    PROVIDE( _sp = . ); ]]);	-- for gd32
	table.insert(script_info, [[  } > RAM]]);
	table.insert(script_info, [[  PROVIDE(__stack = __StackTop);]]);
	table.insert(script_info, [[]]);
	if stackseal_size > 0 then
		table.insert(script_info, [[  .stackseal (NOLOAD) :]]);
		table.insert(script_info, [[  {]]);
		table.insert(script_info, [[    . = ALIGN(8);]]);
		table.insert(script_info, [[    __StackSeal = .;]]);
		table.insert(script_info, [[    . = . + 8;]]);
		table.insert(script_info, [[    . = ALIGN(8);]]);
		table.insert(script_info, [[  } > RAM]]);
	end;
	table.insert(script_info, [[]]);
	table.insert(script_info, [[  ASSERT(__StackLimit >= __HeapLimit, "region RAM overflowed with stack")]]);
	table.insert(script_info, [[}]]);
	table.insert(script_info, [[]]);
	table.insert(script_info, [[/* input sections */]]);
	table.insert(script_info, [[GROUP(libgcc.a libc.a libm.a libnosys.a)]]);
	table.insert(script_info, [[]]);
	return table.concat(script_info, "\n");
end;

local function commit_project(files, message)
	edx:cvs_commit_changes{
		["author"] = {
			["name"] = "EDX";
			["email"] = "www.ed-x.cc";
		};
		["add"] = files;
		["message"] = message or "init";
	};
end;

function gen_default_config(main_file, default_config, default_tool, extras)
	return string.format(
[[project_configs = {
	[".editor"] = {
		opened = {
			"%s";
		};
	};
	[".build"] = {
		config = "%s";
		toolset = "%s";
	};
	[".cmake"] = {
		["argument"] = {
		};
	};
%s};]], main_file, default_config, default_tool, (extras or ""));
end;

local function cmake_create_project(path, name, default_tool, default_config, main_file_content, subsystem)
	local function write_file(file_name, content)
		local cmake_file = io.u8_open(make_path(path, file_name), "w+");
		content = content:gsub("\t", "    ");
		cmake_file:write(content);
		cmake_file:close();
	end;
	local linker_options_win32 = [[if(MSVC)
	add_link_options(/subsystem:windows)
else()
	add_link_options(-mwindows)
endif()]];

	local linker_options = [[# add_link_options(<LINKER_FLAGS>)]];
	if subsystem == "win32" or subsystem == "shared lib" then
		linker_options = linker_options_win32;
	end;
	
	local target_type = "exec";
	if subsystem == "static lib" then
		target_type = "static";
	elseif subsystem == "shared lib" then
		target_type = "shared";
	end;
	local proj_name = name:gsub("%s+", "-"):gsub("[#${}!()]","");
	local target_name = proj_name;
	write_file("CMakeLists.txt",
		make_cmake_list{
			["type"] = target_type;
			["project"] = proj_name;
			["name"] = target_name;
			["nodep"] = false;
			["embeded"] = false;
			["extra"] = linker_options;
		}
	);
	if main_file_content then
		write_file("main.cpp", main_file_content.."\n");
	end;
	if default_tool then
		write_file("settings.lua", gen_default_config("main.cpp", default_config, default_tool));
	end;
end;

local function stm32cube_create_project(path, name, default_tool, default_config, main_file_content, target_device)
	local function write_file(file_name, content)
		local cmake_file = io.u8_open(make_path(path, file_name), "w+");
		content = content:gsub("\t", "    ");
		cmake_file:write(content);
		cmake_file:close();
	end;
	
	local target_type = "exec";
	local proj_name = name:gsub("%s+", "-"):gsub("[#${}!()]","");
	local target_name = proj_name;
	write_file("CMakeLists.txt",
		make_cmake_list{
			["type"] = target_type;
			["project"] = proj_name;
			["name"] = target_name;
			["nodep"] = true;
			["embeded"] = true;
			["extra"] = "";
			["target_link_libs"] = [[${STM32CUBE_SDK}]];
			["target_device"] = target_device;
		}
	);
	pinvoke(
		function()
			local sdk_path = make_path(path, "sdk");
			utils:mkdir(sdk_path);
			local device_info;
			for idx, info in ipairs(EMBEDED_CPU_DB["ST"]) do
				if info[1] == target_device then
					device_info = info;
					break;
				end;
			end;
			if device_info then
				local series_id = target_device:sub(6,7);
				local ld_script_name = nil;
				if target_device:sub(1,5) == "stm32" and series_id == "f1" then
					-- STM32F103xB_FLASH.ld
					local suffix = target_device:sub(11,11);
					local suffix_map = {
						["4"] = "6";
						["6"] = "6";
						["8"] = "b";
						["b"] = "b";
						["c"] = "c";
						["d"] = "e";
						["e"] = "e";
						["f"] = "g";
						["g"] = "g";
					}
					ld_script_name = string.format("stm32%sx%s_flash.ld", target_device:sub(6,9), suffix_map[suffix]);
				elseif target_device:sub(1,5) == "stm32" and series_id == "f3" then
					local suffix = target_device:sub(11,11);
					local suffix_map = {
						["6"] = "8";
						["8"] = "8";
						["b"] = "c";
						["c"] = "c";
						["d"] = "e";
						["e"] = "e";
						["f"] = "g";
						["g"] = "g";
					}
					if target_device:sub(9,9) == '8' then
						ld_script_name = string.format("stm32%sxx_flash.ld", target_device:sub(6,9));
					else
						ld_script_name = string.format("stm32%sx%s_flash.ld", target_device:sub(6,9), suffix_map[suffix]);
					end;
				else
					ld_script_name = string.format("stm32%sxx_flash.ld", target_device:sub(6,9));
				end;
				if ld_script_name then
					local ld_script = make_device_ld_script(device_info);
					write_file("sdk/"..ld_script_name, ld_script);
				end;
			end;
		end
	);

	if main_file_content then
		local src_path = make_path(path, "src");
		utils:mkdir(src_path);
		write_file("src/main.cpp", main_file_content.."\n");
	end;
	if default_tool then
		write_file("settings.lua", gen_default_config("src/main.cpp", default_config, default_tool));
	end;
end;

local function gd32_create_project(path, name, default_tool, default_config, main_file_content, target_device)
	local function write_file(file_name, content)
		local cmake_file = io.u8_open(make_path(path, file_name), "w+");
		content = content:gsub("\t", "    ");
		cmake_file:write(content);
		cmake_file:close();
	end;

	local target_type = "exec";
	local proj_name = name:gsub("%s+", "-"):gsub("[#${}!()]","");
	local target_name = proj_name;
	write_file("CMakeLists.txt",
		make_cmake_list{
			["type"] = target_type;
			["project"] = proj_name;
			["name"] = target_name;
			["nodep"] = true;
			["embeded"] = true;
			["extra"] = "";
			["target_link_libs"] = [[${GD32_SDK}]];
			["target_device"] = target_device;
		}
	);
	pinvoke(
		function()
			local sdk_path = make_path(path, "sdk");
			utils:mkdir(sdk_path);
			local device_info;
			for idx, info in ipairs(EMBEDED_CPU_DB["GigaDevice"]) do
				if info[1] == target_device then
					device_info = info;
					break;
				end;
			end;
			if device_info then
				local ld_script_name = gd32_get_sdk_type_for_device(target_device:sub(1, 8));
				if ld_script_name then
					local ld_script = make_device_ld_script(device_info);
					write_file("sdk/"..ld_script_name.."_flash.ld", ld_script);
				end;
			end;
		end
	);

	if main_file_content then
		local src_path = make_path(path, "src");
		utils:mkdir(src_path);
		write_file("src/main.cpp", main_file_content.."\n");
	end;
	if default_tool then
		write_file("settings.lua", gen_default_config("src/main.cpp", default_config, default_tool));
	end;
end;

local function xhsc_create_project(path, name, default_tool, default_config, main_file_content, target_device)
	local function write_file(file_name, content)
		local cmake_file = io.u8_open(make_path(path, file_name), "w+");
		content = content:gsub("\t", "    ");
		cmake_file:write(content);
		cmake_file:close();
	end;
	
	local target_type = "exec";
	local proj_name = name:gsub("%s+", "-"):gsub("[#${}!()]","");
	local target_name = proj_name;
	write_file("CMakeLists.txt",
		make_cmake_list{
			["type"] = target_type;
			["project"] = proj_name;
			["name"] = target_name;
			["nodep"] = true;
			["embeded"] = true;
			["extra"] = "";
			["target_link_libs"] = [[${XHSC_SDK}]];
			["target_device"] = target_device;
		}
	);
	pinvoke(
		function()
			local sdk_path = make_path(path, "sdk");
			utils:mkdir(sdk_path);
			local device_info;
			for idx, info in ipairs(EMBEDED_CPU_DB["XHSC"]) do
				if info[1] == target_device then
					device_info = info;
					break;
				end;
			end;
			if device_info then
				local series_id = target_device:sub(6,7);
				local ld_script_name = nil;
				if ld_script_name then
					local ld_script = make_device_ld_script(device_info);
					write_file("sdk/"..ld_script_name, ld_script);
				end;
			end;
		end
	);

	if main_file_content then
		local src_path = make_path(path, "src");
		utils:mkdir(src_path);
		write_file("src/main.cpp", main_file_content.."\n");
	end;
	if default_tool then
		write_file("settings.lua", gen_default_config("src/main.cpp", default_config, default_tool));
	end;
end;

local function airmcu_create_project(path, name, default_tool, default_config, main_file_content, target_device)
	local function write_file(file_name, content)
		local cmake_file = io.u8_open(make_path(path, file_name), "w+");
		content = content:gsub("\t", "    ");
		cmake_file:write(content);
		cmake_file:close();
	end;
	
	local target_type = "exec";
	local proj_name = name:gsub("%s+", "-"):gsub("[#${}!()]","");
	local target_name = proj_name;
	write_file("CMakeLists.txt",
		make_cmake_list{
			["type"] = target_type;
			["project"] = proj_name;
			["name"] = target_name;
			["nodep"] = true;
			["embeded"] = true;
			["extra"] = "";
			["target_link_libs"] = [[${AIRMCU_SDKS}]];
			["target_device"] = target_device;
		}
	);
	pinvoke(
		function()
			local sdk_path = make_path(path, "sdk");
			utils:mkdir(sdk_path);
			local device_info;
			for idx, info in ipairs(EMBEDED_CPU_DB["AirMCU"]) do
				if info[1] == target_device then
					device_info = info;
					break;
				end;
			end;
			if device_info then
				local target_device_type = target_device:sub(1,6);
				local ld_script_name = nil;
				if target_device_type == "air32f" then
					local series_id = target_device:sub(6,7);
					if series_id == "f1" then
						-- Air32F103xB_FLASH.ld
						local suffix = target_device:sub(11,11);
						local suffix_map = {
							["b"] = "b";
							["c"] = "b";
							["d"] = "e";
							["e"] = "e";
							["f"] = "g";
							["g"] = "g";
						}
						ld_script_name = string.format("air32%sx%s_flash.ld", target_device:sub(6,9), suffix_map[suffix]);
					else
						ld_script_name = string.format("air32%sxx_flash.ld", target_device:sub(6,9));
					end;
				else
					ld_script_name = string.format("%sxx_flash.ld", target_device);
				end;
				if ld_script_name then
					local ld_script = make_device_ld_script(device_info);
					write_file("sdk/"..ld_script_name, ld_script);
				end;
			end;
		end
	);

	if main_file_content then
		local src_path = make_path(path, "src");
		utils:mkdir(src_path);
		write_file("src/main.cpp", main_file_content.."\n");
	end;
	if default_tool then
		write_file("settings.lua", gen_default_config("src/main.cpp", default_config, default_tool));
	end;
end;

local function pico_create_project(path, name, default_tool, default_config, main_file_content, target_device)
	local function write_file(file_name, content)
		local cmake_file = io.u8_open(make_path(path, file_name), "w+");
		content = content:gsub("\t", "    ");
		cmake_file:write(content);
		cmake_file:close();
	end;

	local target_type = "exec";
	local proj_name = name:gsub("%s+", "-"):gsub("[#${}!()]","");
	local target_name = proj_name;
	write_file("CMakeLists.txt",
		make_cmake_list{
			["type"] = target_type;
			["project"] = proj_name;
			["name"] = target_name;
			["nodep"] = true;
			["embeded"] = "pico";
            ["extra"] = "# pico_generate_pio_header(${TARGET_NAME} ${CMAKE_CURRENT_LIST_DIR}/src/${TARGET_NAME}.pio)";
            ["target_link_libs"] = {
                "pico_multicore";
                "pico_stdlib";
                "pico_unique_id";
                "hardware_pio";
            };
			["target_device"] = nil;
		}
	);

	if main_file_content then
		local src_path = make_path(path, "src");
		utils:mkdir(src_path);
		write_file("src/main.cpp", main_file_content.."\n");
	end;
	if default_tool then
		write_file("settings.lua", gen_default_config("src/main.cpp", default_config, default_tool));
	end;
end;

local function xmake_create_project(path, name, default_tool, default_config, template, on_finished_handler)
	local function write_file(file_name, content)
		local cmake_file = io.u8_open(make_path(path, file_name), "w+");
		cmake_file:write(content);
		cmake_file:close();
	end;
	local cmds = {[["]]..XMAKE_PATHS[1]..[[" create -P "]]..path..[[" -t "]]..template..[[" "]]..name..[["]]};
	local tasks = {
		desc = "XMake create project";
		["cmds"] = cmds;
		on_output = function(cmd_idx, output_line) end;
		on_command_begin = function(cmd_idx)
			print(cmds[cmd_idx]);
		end;
		on_command_end = function(cmd_idx, ret_code)
		end;
		on_finished = function(success)
			if on_finished_handler then
				if success then
					if default_tool then
						write_file("settings.lua", gen_default_config("src/main.cpp", default_config, default_tool));
					end;
				end;
				on_finished_handler(success);
			end;
		end;
		encoding = "ansi";
	};
	edx:do_build(tasks)
end;

function project_wizard:on_init()
	self:init_icons({ "group unfolded", "group folded", "terminal", "exe file", "lib file", "dll file", "register"});
	local project_types = {
		{
			{LANG("dialog/project_wizard/type/cmake"),"native"},
			{LANG("dialog/project_wizard/type/console"), "console", 3},
			{LANG("dialog/project_wizard/type/win32"), "win32", 4},
			{LANG("dialog/project_wizard/type/static"), "static lib", 5},
			{LANG("dialog/project_wizard/type/shared"), "shared lib", 6},
		};
	};
	local has_wsl = false;
	local has_esp32 = ESP32_IDF_TOOLSETS ~= nil and #ESP32_IDF_TOOLSETS > 0;
	local has_bouffalo = BL_TOOLSETS ~= nil and #BL_TOOLSETS > 0;
	for idx, tool in ipairs(TOOLSETS) do
		if tool["target"] == "wsl" then
			if not has_wsl then
				self.__wsl_toolset = tool.name;
			end;
			has_wsl = true;
		end;
	end;
	if XMAKE_PATHS[1] then
		table.insert(project_types,	{
				{LANG("dialog/project_wizard/type/xmake"), "xmake"},
				{LANG("dialog/project_wizard/type/console"), "console", 3},
				{LANG("dialog/project_wizard/type/static"), "static", 5},
				{LANG("dialog/project_wizard/type/shared"), "shared", 6},
				{
					{"Qt"};
					{
						{"Qt.Console", "qt.console", 3},
						{"Qt.QuickApp", "qt.quickapp", 4},
						{"Qt.QuickApp_static", "qt.quickapp_static", 4},
						{"Qt.Static", "qt.static", 4},
						{"Qt.Shared", "qt.shared", 4},
						{"Qt.WidgetApp", "qt.widgetapp", 4},
						{"Qt.WidgetApp_Static", "qt.widgetapp_static", 4},
						
					};
				}
		});
	end;
	if has_wsl then
		table.insert(project_types, {
				{LANG("dialog/project_wizard/type/wsl"), "wsl"},
				{LANG("dialog/project_wizard/type/console"), "console", 3},
				{LANG("dialog/project_wizard/type/static"), "static lib", 5},
				{LANG("dialog/project_wizard/type/shared"), "shared lib", 6},
		});
	end;
	if has_esp32 then
		local all_esp_targets = {};
		for idx, info in ipairs(ESP32_IDF_TOOLSETS) do
			table.append(all_esp_targets, table.unpack(info.targets));
		end;
		all_esp_targets = table.unique(all_esp_targets);
		if all_esp_targets and #all_esp_targets > 0 then
			table.sort(all_esp_targets);
			local esp_project_info = {
				{LANG("dialog/project_wizard/type/esp-idf"), "esp-idf"},
			};
			for idx, target in ipairs(all_esp_targets) do
				table.insert(esp_project_info, {target:upper(), target, 7});
			end;
			table.insert(project_types,	esp_project_info);
		end;
	end;
	if has_bouffalo then
		local all_bl_targets = {};
		for idx, info in ipairs(BL_TOOLSETS) do
			table.append(all_bl_targets, table.unpack(info.targets));
		end;
		all_bl_targets = table.unique(all_bl_targets);
		if all_bl_targets and #all_bl_targets > 0 then
			table.sort(all_bl_targets);
			local bl_project_info = {
				{LANG("dialog/project_wizard/type/bouffalo_sdk"), "bouffalo_sdk"},
			};
			for idx, target in ipairs(all_bl_targets) do
				table.insert(bl_project_info, {target:upper(), target, 7});
			end;
			table.insert(project_types,	bl_project_info);
		end;
	end;

	do
		local stm32cube_projects_templates = {
			["stm32f0xx"] = {
				{"STM32F0xx"};
				{};
			};
			["stm32f1xx"] = {
				{"STM32F1xx"};
				{};
			};
			["stm32f2xx"] = {
				{"STM32F2xx"};
				{};
			};
			["stm32f3xx"] = {
				{"STM32F3xx"};
				{};
			};
			["stm32f4xx"] = {
				{"STM32F4xx"};
				{};
			};
			["stm32f7xx"] = {
				{"STM32F7xx"};
				{};
			};
			["stm32h5xx"] = {
				{"STM32H5xx"};
				{};
			};
			["stm32h7xx"] = {
				{"STM32H7xx"};
				{};
			};
			["stm32l0xx"] = {
				{"STM32L0xx"};
				{};
			};
			["stm32l1xx"] = {
				{"STM32L1xx"};
				{};
			};
			["stm32l4xx"] = {
				{"STM32L4xx"};
				{};
			};
			["stm32l5xx"] = {
				{"STM32L5xx"};
				{};
			};
			["stm32g0xx"] = {
				{"STM32G0xx"};
				{};
			};
			["stm32g4xx"] = {
				{"STM32G4xx"};
				{};
			};
			["stm32u5xx"] = {
				{"STM32U5xx"};
				{};
			};
		};

		local st_cpus = EMBEDED_CPU_DB["ST"] or {};
		for idx, info in ipairs(st_cpus) do
			local cpu_name = info[1];
			local cpu_prefix = cpu_name:sub(1,7):lower().."xx";
			local info = stm32cube_projects_templates[cpu_prefix];
			if info then
				table.insert(info[2], {cpu_name:upper(), cpu_name, 7});
			end;
		end;
		pinvoke(
			function()
				local stm32cube_projects = {{"STM32Cube", "stm32cube"};};
				local available_cube_targets = {};
				for idx, toolset in ipairs(TOOLSETS) do
					if toolset["type"] == "STM32Cube" then
						for ver, sdk in pairs(toolset["sdks_info"]) do
							table.insert(available_cube_targets, ver:sub(1, 9));
						end;
					end;
				end;
				table.sort(available_cube_targets);
				available_cube_targets = table.unique(available_cube_targets);
				for idx, target in ipairs(available_cube_targets) do
					local template = stm32cube_projects_templates[target];
					if template then
						table.insert(stm32cube_projects, template);
					end;
				end;
				if #stm32cube_projects > 1 then
					table.insert(project_types, stm32cube_projects);
				end;
			end
		);
	end;
	
	do
		local gd32_projects_templates = {
			["gd32a5xx"] = {
				{"GD32A5xx"};
				{};
			};
			["gd32c1xx"] = {
				{"GD32C1xx"};
				{};
			};
			["gd32e1xx"] = {
				{"GD32E1xx"};
				{};
			};
			["gd32e2xx"] = {
				{"GD32E2xx"};
				{};
			};
			["gd32e5xx"] = {
				{"GD32E5xx"};
				{};
			};
			["gd32f1xx"] = {
				{"GD32F1xx"};
				{};
			};
			["gd32f2xx"] = {
				{"GD32F2xx"};
				{};
			};
			["gd32f3xx"] = {
				{"GD32F3xx"};
				{};
			};
			["gd32f4xx"] = {
				{"GD32F4xx"};
				{};
			};
			["gd32h7xx"] = {
				{"GD32H7xx"};
				{};
			};
			["gd32l2xx"] = {
				{"GD32L2xx"};
				{};
			};
		};

		local gd_cpus = EMBEDED_CPU_DB["GigaDevice"] or {};
		for idx, info in ipairs(gd_cpus) do
			local cpu_name = info[1];
			local cpu_prefix = cpu_name:sub(1,6):lower().."xx";
			local info = gd32_projects_templates[cpu_prefix];
			if info then
				table.insert(info[2], {cpu_name:upper(), cpu_name, 7});
			end;
		end;
		pinvoke(
			function()
				local gd32_projects = {{"GigaDevice", "gd32"};};
				local available_gd32_targets = {};
				for idx, toolset in ipairs(TOOLSETS) do
					if toolset["type"] == "GD32" then
						for ver, sdk in pairs(toolset["sdks_info"]) do
							table.insert(available_gd32_targets, ver:sub(1, 6).."xx");
						end;
					end;
				end;
				table.sort(available_gd32_targets);
				available_gd32_targets = table.unique(available_gd32_targets);
				for idx, target in ipairs(available_gd32_targets) do
					local template = gd32_projects_templates[target];
					if template then
						table.insert(gd32_projects, template);
					end;
				end;
				if #gd32_projects > 1 then
					table.insert(project_types, gd32_projects);
				end;
			end
		);
	end;
	
	do
		local xhsc_projects_templates = {
			["hc32f460"] = {
				{"HC32F460"};
				{};
			};
		};

		local xhsc_cpus = EMBEDED_CPU_DB["XHSC"] or {};
		for idx, info in ipairs(xhsc_cpus) do
			local cpu_name = info[1];
			local cpu_prefix = cpu_name:sub(1,8):lower();
			local info = xhsc_projects_templates[cpu_prefix];
			if info then
				table.insert(info[2], {cpu_name:upper(), cpu_name, 7});
			end;
		end;
		pinvoke(
			function()
				local xhsc_projects = {{"XHSC", "xhsc"};};
				local available_xhsc_targets = {};
				for idx, toolset in ipairs(TOOLSETS) do
					if toolset["type"] == "XHSC" then
						for ver, sdk in pairs(toolset["sdks_info"]) do
							table.insert(available_xhsc_targets, ver:sub(1, 8));
						end;
					end;
				end;
				table.sort(available_xhsc_targets);
				available_xhsc_targets = table.unique(available_xhsc_targets);
				for idx, target in ipairs(available_xhsc_targets) do
					local template = xhsc_projects_templates[target];
					if template then
						table.insert(xhsc_projects, template);
					end;
				end;
				if #xhsc_projects > 1 then
					table.insert(project_types, xhsc_projects);
				end;
			end
		);
	end;
	
	do
		local airmcu_projects_templates = {
			["air001xx"] = {
				{"AIR001xx"};
				{};
			};
			["air32f1xx"] = {
				{"AIR32F1xx"};
				{};
			};
		};

		local air_cpus = EMBEDED_CPU_DB["AirMCU"] or {};
		for idx, info in ipairs(air_cpus) do
			local cpu_name = info[1];
			local cpu_prefix;
			if cpu_name:sub(1,6) == "air32f" then
				cpu_prefix = cpu_name:sub(1,7):lower().."xx";
			else
				cpu_prefix = cpu_name.."xx";
			end;
			local info = airmcu_projects_templates[cpu_prefix];
			if info then
				table.insert(info[2], {cpu_name:upper(), cpu_name, 7});
			end;
		end;
		pinvoke(
			function()
				local airmcu_projects = {{"AirMCU", "airmcu"};};
				local available_airmcu_targets = {};
				for idx, toolset in ipairs(TOOLSETS) do
					if toolset["type"] == "AirMCU" then
						for ver, sdk in pairs(toolset["sdks_info"]) do
							table.insert(available_airmcu_targets, ver:sub(1, 9));
						end;
					end;
				end;
				table.sort(available_airmcu_targets);
				available_airmcu_targets = table.unique(available_airmcu_targets);
				for idx, target in ipairs(available_airmcu_targets) do
					local template = airmcu_projects_templates[target];
					if template then
						table.insert(airmcu_projects, template);
					end;
				end;
				if #airmcu_projects > 1 then
					table.insert(project_types, airmcu_projects);
				end;
			end
		);
	end;

	do
		local pico_projects = {
			{"Raspberry Pi", "rbpi"};
			{"Pico 2040", "rp2040", 7},
		};

		pinvoke(
			function()
				local toolset = pico_find_toolset_for_device("pico 2040");
				if toolset then
					table.insert(project_types, pico_projects);
				end;
			end
		);
	end;

	if ZEPHYR_TOOLSET then
		local zephyr_projects = {
			{"Zephyr", "zephyr"};
			{"App", "zephyr", 7},
		};
		table.insert(project_types, zephyr_projects);
	end;

	self:init_wizard(project_types);
	self:select("native", "console");
end;

function project_wizard:on_project_type_changed(name, group)
	-- print("on_project_type_changed(", name, ",", group, ")");
	-- self:set_info("INFO "..name..","..group);
	return 1;
end;

function project_wizard:on_dismissed()
	return 1;
end;

function project_wizard:do_create(info)
	local file_content_map = {
		["wsl"] = {
			["console"] = [[#include <stdio.h>
int main() {
	printf("hello world\n");
	return 0;
}]];
			["shared lib"] = [[#include <stdio.h>
int foo() {
	printf("hello world\n");
	return 0;
}]];
			["static lib"] = [[#include <stdio.h>
int foo() {
	printf("hello world\n");
	return 0;
}]];
		};
		["native"] = {
			["console"] = [[#include <stdio.h>
int main() {
	printf("hello world\n");
	return 0;
}]];
			["win32"] = [[#include <windows.h>
int WINAPI WinMain(HINSTANCE hinst, HINSTANCE hPrevInst, LPSTR lpCmd, int nShow) {
	MessageBoxA(0, "hello world\n", 0, MB_OK);
	return 0;
}]];
			["shared lib"] = [[#include <windows.h>
BOOL WINAPI DllMain(
	HINSTANCE hinstDLL,  // handle to DLL module
	DWORD fdwReason,     // reason for calling function
	LPVOID lpReserved )  // reserved
{
	// Perform actions based on the reason for calling.
	switch( fdwReason ) {
		case DLL_PROCESS_ATTACH:
			// Initialize once for each new process.
			// Return FALSE to fail DLL load.
			break;

		case DLL_THREAD_ATTACH:
			// Do thread-specific initialization.
			break;

		case DLL_THREAD_DETACH:
			// Do thread-specific cleanup.
			break;

		case DLL_PROCESS_DETACH:
			// Perform any necessary cleanup.
			break;
	}
	return TRUE;  // Successful DLL_PROCESS_ATTACH.
}]];
			["static lib"] = [[#include <stdio.h>
int foo() {
	printf("hello world\n");
	return 0;
}]];
		};
	};
	
	local proj_type = info["type"];
	local proj_type_group = info["group"];
	local proj_name = info["name"];
	local proj_path = info["path"];
	
	local create_git_repo = function()
		if info["create_git_repo"] ~= true then
			return;
		end;
		print("Create GIT repository...");
		local git_ignore_file = make_path(proj_path, ".gitignore");
		local git_ignore_content = ".edx/\nsettings.lua\n";
		if utils:file_time(git_ignore_file) then
			local old_file = load_file(git_ignore_file);
			if old_file:sub(#old_file) ~= "\n" then
				git_ignore_content = old_file .. "\n# EDX files\n" .. git_ignore_content;
			else
				git_ignore_content = old_file .. "# EDX files\n" .. git_ignore_content;
			end;
		end;
		write_file(git_ignore_file, git_ignore_content);
		edx:cvs_init_repo("GIT", proj_path);
	end;
	local switch_to_new_proj = function(proj_path, commit_files)
		project_wizard:dismiss();
		lsp:terminate();
		menu_bar:enable_cmake_menu_items();
		xws_mgr:load_xws(proj_path);
		if info["create_git_repo"] == true then
			commit_project(commit_files);
		end;
	end;
	local toolset = TOOLSETS[1];
	local default_toolset = toolset and toolset.name;
	if toolset and toolset["type"] == "msvc" then
		default_toolset = toolset.name.."|"..toolset.version.."|"..toolset.sdk;
	end;
	
	if proj_type_group == "xmake" then
		if utils:file_time(make_path(proj_path, "xmake.lua")) then
			self:show_tip("path", LANG("dialog/project_wizard/project_exists"));
			return 0;
		end;
		if utils:mkdir(proj_path) then
			self:show_tip("path", LANG("dialog/project_wizard/can_not_create_project_folder"));
			return 0;
		end;

		xmake_create_project(proj_path, proj_name, default_toolset, "Debug", proj_type,
			function(success)
				if success then
					project_wizard:dismiss();
					lsp:terminate();
					menu_bar:enable_cmake_menu_items();
					create_git_repo();
					xws_mgr:load_xws(proj_path);
					if info["create_git_repo"] == true then
						local files = utils:list_dir(make_path(proj_path, "src/*"), true) or {};
						for idx, name in ipairs(files) do
							files[idx] = "src/"..name;
						end;
						commit_project{
							".gitignore";
							"xmake.lua";
							table.unpack(files);
						};
					end;
				else
					project_wizard:enable(true);
					self:show_tip("path", LANG("dialog/project_wizard/can_not_create_project"));
				end;
			end
		);
		return 1;
	end;
	
	if utils:file_time(make_path(proj_path, "CMakeLists.txt")) then
		self:show_tip("path", LANG("dialog/project_wizard/project_exists"));
		return 0;
	end;
	if utils:mkdir(proj_path) then
		self:show_tip("path", LANG("dialog/project_wizard/can_not_create_project_folder"));
		return 0;
	end;
	if proj_type_group == "esp-idf" then
		project_wizard:enable(false);
		esp_idf_create_project(proj_path, proj_name, proj_type,
			function(success, toolset_name, main_file, extras)
				if success then
					write_file(make_path(proj_path, "settings.lua"), gen_default_config(main_file, "Debug", toolset_name, extras));
					create_git_repo();
					switch_to_new_proj(
						proj_path,
						{
							".gitignore";
							"CMakeLists.txt";
							"sdkconfig";
							"main/CMakeLists.txt";
							main_file;
						}
					);
				else
					project_wizard:enable(true);
					self:show_tip("path", LANG("dialog/project_wizard/can_not_create_project"));
				end;
			end
		);
		return 1;
	end;
	if proj_type_group == "bouffalo_sdk" then
		project_wizard:enable(false);
		bl_create_project(proj_path, proj_name, proj_type,
			function(success, toolset_name, main_file, extras)
				if success then
					write_file(make_path(proj_path, "settings.lua"), gen_default_config(main_file, "Debug", toolset_name, extras));
					create_git_repo();
					switch_to_new_proj(
						proj_path,
						{
							".gitignore";
							"CMakeLists.txt";
							"proj.conf";
							main_file;
						}
					);
				else
					project_wizard:enable(true);
					self:show_tip("path", LANG("dialog/project_wizard/can_not_create_project"));
				end;
			end
		);
		return 1;
	end;
	if proj_type_group == "zephyr" then
		project_wizard:enable(false);
		zephyr_create_project(proj_path, proj_name, proj_type,
			function(success, toolset_name, main_file, extras)
				if success then
					write_file(make_path(proj_path, "settings.lua"), gen_default_config(main_file, "Debug", toolset_name, extras));
					create_git_repo();
					switch_to_new_proj(
						proj_path,
						{
							".gitignore";
							"CMakeLists.txt";
							"prj.conf";
							main_file;
						}
					);
				else
					project_wizard:enable(true);
					self:show_tip("path", LANG("dialog/project_wizard/can_not_create_project"));
				end;
			end
		);
		return 1;
	end;
	if proj_type_group == "wsl" then
		cmake_create_project(proj_path, proj_name, self.__wsl_toolset, "Debug", file_content_map[proj_type_group][proj_type], proj_type);
		default_toolset = self.__wsl_toolset;
	elseif proj_type_group == "native" then
		cmake_create_project(proj_path, proj_name, default_toolset, "Debug", file_content_map[proj_type_group][proj_type], proj_type);
	elseif proj_type_group == "stm32cube" then
		local toolset = stm32cube_find_toolset_for_device(proj_type);
		if toolset == nil then
			print("Can't find the STM32Cube sdk!");
			return;
		end;
		local main_file = table.concat(
			{
				[[#include <stdio.h>]];
				[[]];
				[[int main() {]];
				[[	printf("hello world!\n");]];
				[[	return 0;]];
				[[}]];
				[[]];
			},
			"\n"
		);
		stm32cube_create_project(proj_path, proj_name, toolset, "Debug", main_file, proj_type);
	elseif proj_type_group == "gd32" then
		local toolset = gd32_find_toolset_for_device(proj_type);
		if toolset == nil then
			print("Can't find the GD32 sdk!");
			return;
		end;
		local main_file = table.concat(
			{
				[[#include <stdio.h>]];
				[[]];
				[[int main() {]];
				[[	printf("hello world!\n");]];
				[[	return 0;]];
				[[}]];
				[[]];
			},
			"\n"
		);
		gd32_create_project(proj_path, proj_name, toolset, "Debug", main_file, proj_type);
	elseif proj_type_group == "airmcu" then
		local toolset = airmcu_find_toolset_for_device(proj_type);
		if toolset == nil then
			print("Can't find the AirMCU sdk!");
			return;
		end;
		local main_file = table.concat(
			{
				[[#include <stdio.h>]];
				[[]];
				[[int main() {]];
				[[	printf("hello world!\n");]];
				[[	return 0;]];
				[[}]];
				[[]];
			},
			"\n"
		);
		airmcu_create_project(proj_path, proj_name, toolset, "Debug", main_file, proj_type);
	elseif proj_type_group == "rbpi" then
		local toolset = pico_find_toolset_for_device(proj_type);
		if toolset == nil then
			print("Can't find the Raspberry Pi Pico SDK!");
			return;
		end;
		local main_file = table.concat(
			{
				[[#include <stdio.h>]];
				[[#include "pico/stdlib.h"]];
				[[]];
				[[int main() {]];
				[[	stdio_init_all();]];
				[[	while (true) {]];
				[[		printf("Hello, world!\n");]];
				[[		sleep_ms(1000);]];
				[[	}]];
				[[}]];
				[[]];
			},
			"\n"
		);
		pico_create_project(proj_path, proj_name, toolset, "Debug", main_file, proj_type);
	elseif proj_type_group == "xhsc" then
		local toolset = xhsc_find_toolset_for_device(proj_type);
		if toolset == nil then
			print("Can't find the XHSC sdk!");
			return;
		end;
		local main_file = table.concat(
			{
				[[#include <stdio.h>]];
				[[]];
				[[int main() {]];
				[[	printf("hello world!\n");]];
				[[	return 0;]];
				[[}]];
				[[]];
			},
			"\n"
		);
		xhsc_create_project(proj_path, proj_name, toolset, "Debug", main_file, proj_type);
	end;

	create_git_repo();
	switch_to_new_proj(
		proj_path,
		{
			".gitignore";
			"CMakeLists.txt";
			"main.cpp";
		}
	);
	return 1;
end;
